/*****************************************************************
                       НЕОФИЦИАЛЬНЫЕ СПЕЦИФИКАЦИИ
Критические данные, используемые DENUVO для формирования TableData
Размерность оперирования данными при чтении: DWORD (кроме OSPlatformId = WORD)
Полагаемая разбивка прочитанных данных в TableData: BYTE
Для CPUID при EAX = 1 критичные регистры только: EBX, ECX
VER: 0,97 (07.01.2017)
(С)ELF, 2017
https://exelab.ru/F/index.php?action=vthread&forum=13&topic=19719
https://exelab.ru/F/PAunlock/ :)
*****************************************************************/

// __________________ code definitions __________________ //
// __ KUSER_SHARED_DATA offsets
#define defKUSER_CryptoExponen_offset 0x023C
#define defKUSER_NtMajorVersion_offset 0x026C
#define defKUSER_NtMinorVersion_offset 0x0270
#define defKUSER_ProcessorFeatures_offset 0x0274
#define defKUSER_TimeSlip_offset 0x02BC
#define defKUSER_NumberOfPhysicalPages_offset 0x02E8
#define defKUSER_NtSystemRoot_offset 0x030

// __ Process Enviroment Block (PEB) offsets
#define defPEB_OSMajorVersion_offset  0x118
#define defPEB_OSMinorVersion_offset  0x11C
#define defPEB_ImageSubSystemMajorVersion_offset     0x12C
#define defPEB_ImageSubSystemMinorVersion_offset     0x130
#define defPEB_NumberOfProcessors_offset  0x0B8
#define defPEB_OSPlatformId_short_offset 0x122 

    #define MM_SHARED_USER_DATA_VA      0x7FFE0000
    #define USER_SHARED_DATA ((KUSER_SHARED_DATA * const)MM_SHARED_USER_DATA_VA)



//++++++++++++++++++++++ structs consist of ++++++++++++++++++++++ //
typedef struct D_CPUID_REGISTER_enumerator
{
	unsigned long reg_EAX;
	unsigned long reg_EBX;
	unsigned long reg_ECX;
	unsigned long reg_EDX;
}REGISTER_enumerator, *pREGISTER_enumerator; //x86 регистры CPU

typedef struct D_CPUID_lic_block
{
	REGISTER_enumerator EAX_1; //Additional Information / Feature Information
	REGISTER_enumerator EAX_80000002; //Processor Brand String
	REGISTER_enumerator EAX_80000003; //Processor Brand String продолжение
	REGISTER_enumerator EAX_80000004; //Processor Brand String окончаение
}CPUID_licence_stack, *pCPUID_licence_stack; //CPUID stack

typedef struct D_KUSER_SHARED_DATA_lic_block
{
	//critical aviable data
	WCHAR NtSystemRoot[MAX_PATH]; // в 99% случаев "C:\Windows" в unicode 
	ULONG NtMajorVersion; //Major-значение (1 - для Windows 7)
    ULONG NtMinorVersion; //Minor-значение (6 - для Windows 7)
	UCHAR ProcessorFeatures[64]; //перечиляются биты возможностей
	ULONG NumberOfPhysicalPages; //кол-во physpages (например: 0x000FFF4E)
	
	//n/a, but access. Не используемые, но читаемые
	 ULONG CryptoExponent;
	 ULONG TimeSlip;
}KUSER_SHARED_DATA_licence_stack, *pKUSER_SHARED_DATA_licence_stack; //KUSER_SHARED_DATA stack

typedef struct D_IMAGE_DATA_DIRECTORY_lic_block
{
	WCHAR Service_DllName[MAX_PATH]; //service data - dll имя
	
	//critical IMAGE_DATA_DIRECTORY data (из IMAGE_OPTIONAL_HEADER)
	ULONG ExportDirectoryVA;
	ULONG ExportDirectorySize;

	ULONG ImportDirectoryVA;
	ULONG ImportDirectorySize;

	ULONG ResourseDirectoryVA;
	ULONG ResourseDirectorySize;

	ULONG ExceptionDirectoryVA;
	ULONG ExceptionDirectorySize;

	ULONG RelocationDirectoryVA;
	ULONG RelocationDirectorySize;

	ULONG DebugDirectoryVA;
	ULONG DebugDirectorySize;
}IMAGE_DATA_DIRECTORY_licence_stack, *pIMAGE_DATA_DIRECTORY_licence_stack; //IMAGE_DATA_DIRECTORY stack

typedef struct D_PEB_lic_block
{
  ULONG                   OSMajorVersion;
  ULONG                   OSMinorVersion;
  ULONG                   ImageSubSystemMajorVersion;
  ULONG                   ImageSubSystemMinorVersion;
  ULONG                   NumberOfProcessors;
  USHORT				  OSPlatformId_short;
}PEB_licence_stack, *pPEB_licence_stack; //Process Enviroment Block (PEB) stack

//+-+-+-+-+-+-+-+-+-+-+-+-+ main struct +-+-+-+-+-+-+-+-+-+-+-+-+ //
typedef struct D_Licence_RAW_TABLE_DATA
{
CPUID_licence_stack CPUID_stack; //CPUID
KUSER_SHARED_DATA_licence_stack KUSER_SHARED_DATA_stack; //KUSER_SHARED_DATA
IMAGE_DATA_DIRECTORY_licence_stack IDD_system_dll_1; //IMAGE_DATA_DIRECTORY in dll #1 (def: ntdll)
IMAGE_DATA_DIRECTORY_licence_stack IDD_system_dll_2; //IMAGE_DATA_DIRECTORY in dll #2 (def: kernel32)
IMAGE_DATA_DIRECTORY_licence_stack IDD_system_dll_3; //IMAGE_DATA_DIRECTORY in dll #3 (any system library)
PEB_licence_stack PEB_stack; // Process Enviroment Block (PEB)
}StRaw_licence_data, *pSt_Raw_licence_data; //DENUVO raw lic strucure
